package org.apithefire.servlet.pathmap;

import java.util.ArrayList;
import java.util.List;

import javax.servlet.Filter;

import org.apithefire.servlet.util.ServletUtil;
import org.apithefire.util.lang.Objects;
import org.apithefire.util.lang.Path;

class FilterMap {

	private static abstract class FilterHolder {
		
		private Filter filter;
		
		public FilterHolder(Filter filter) {
			this.filter = Objects.nonNull(filter);
		}
		
		public Filter getFilter() {
			return filter;
		}
		
		public abstract boolean matches(Path path, Path suffix);
		
	}
	
	private static class ExactFilterHolder extends FilterHolder {
		
		private Path exactPath;
		
		public ExactFilterHolder(Path exactPath, Filter filter) {
			super(filter);
			this.exactPath = Objects.nonNull(exactPath);
		}
		
		@Override
		public boolean matches(Path path, Path suffix) {
			return exactPath.equals(path);
		}
		
	}
	
	private static class PrefixFilterHolder extends FilterHolder {
		
		private Path prefix;
		
		public PrefixFilterHolder(Path prefix, Filter filter) {
			super(filter);
			this.prefix = Objects.nonNull(prefix);
		}

		@Override
		public boolean matches(Path path, Path suffix) {
			return path.startsWith(prefix);
		}
		
	}
	
	private static class SuffixFilterHolder extends FilterHolder {
		
		private Path suffix;
		
		public SuffixFilterHolder(Path suffix, Filter filter) {
			super(filter);
			this.suffix = Objects.nonNull(suffix);
		}

		@Override
		public boolean matches(Path path, Path suffix) {
			return suffix.endsWith(this.suffix);
		}
		
	}
	
	private final List<FilterHolder> filterHolders;
	
	public FilterMap() {
		filterHolders = new ArrayList<FilterHolder>();
	}
	
	public List<Filter> getMatches(Path path) {
		List<Filter> filters = new ArrayList<Filter>(filterHolders.size());
		Path suffix = ServletUtil.splitLastComponentAsSuffix(path);
		
		for (FilterHolder filterHolder : filterHolders) {
			if (filterHolder.matches(path, suffix)) {
				filters.add(filterHolder.getFilter());
			}
		}
		return filters;
	}
	
	public void putExact(Path exactPath, Filter filter) {
		filterHolders.add(new ExactFilterHolder(exactPath, filter));
	}
	
	public void putPrefix(Path prefix, Filter filter) {
		filterHolders.add(new PrefixFilterHolder(prefix, filter));
	}
	
	public void putSuffix(Path suffix, Filter filter) {
		filterHolders.add(new SuffixFilterHolder(suffix, filter));
	}
	
}
