package org.apithefire.servlet.pathmap;

import java.io.IOException;
import java.util.Collections;
import java.util.List;
import java.util.ListIterator;
import java.util.Map;

import javax.servlet.Filter;
import javax.servlet.FilterChain;
import javax.servlet.Servlet;
import javax.servlet.ServletContext;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletRequestWrapper;
import javax.servlet.http.HttpServletResponse;

import org.apithefire.servlet.dynamic.DynamicServlet;
import org.apithefire.servlet.util.ServletUtil;
import org.apithefire.util.lang.Objects;
import org.apithefire.util.lang.Path;

public class PathMapServlet extends HttpServlet implements DynamicServlet {
	
	private static class ServletMatchFilterChain implements FilterChain {
		
		private final ServletMatch servletMatch;
		
		public ServletMatchFilterChain(ServletMatch servletMatch) {
			this.servletMatch = Objects.nonNull(servletMatch);
		}

		public void doFilter(ServletRequest request,
				ServletResponse response) throws IOException,
				ServletException {
			HttpServletRequest requestWrapper = new ServletMatchRequestWrapper(
					servletMatch, (HttpServletRequest) request);
			servletMatch.getServlet()
					.service(requestWrapper, response);
		}
		
	}
	
	private static class ServletMatchRequestWrapper extends
			HttpServletRequestWrapper {
		
		private final ServletMatch servletMatch;

		public ServletMatchRequestWrapper(ServletMatch servletMatch,
				HttpServletRequest request) {
			super(request);
			this.servletMatch = Objects.nonNull(servletMatch);
		}

		@Override
		public String getServletPath() {
			Path servletPath = servletMatch.getServletPath();
			
			return "/" + servletPath.trim().toString();
		}

		@Override
		public String getPathInfo() {
			Path pathInfo = servletMatch.getPathInfo();
			
			if (pathInfo.isEmpty()) {
				return null;
			}
			return "/" + pathInfo.trim().toString();
		}
		
	}
	
	private static class FilterFilterChain implements FilterChain {
		
		private final Filter filter;
		private final FilterChain filterChain;
		
		public FilterFilterChain(Filter filter, FilterChain filterChain) {
			this.filter = Objects.nonNull(filter);
			this.filterChain = Objects.nonNull(filterChain);
		}

		public void doFilter(ServletRequest request,
				ServletResponse response) throws IOException,
				ServletException {
			filter.doFilter(request, response, filterChain);
		}
		
	}

	private static final FilterChain EMPTY_CHAIN = new FilterChain() {
		public void doFilter(ServletRequest request,
				ServletResponse response) throws IOException,
				ServletException {
		}
	};
	
	private final ServletMap servletMap = new ServletMap();
	private final FilterMap filterMap = new FilterMap();
	
	public void init(ServletContext servletContext) throws ServletException {
	}

	public Map<String, String> getInitParamMap() {
		return Collections.emptyMap();
	}
	
	public synchronized void addExactServlet(Path exactPath, Servlet servlet) {
		servletMap.putExact(exactPath, servlet);
	}
	
	public synchronized void addPrefixServlet(Path prefix, Servlet servlet) {
		servletMap.putPrefix(prefix, servlet);
	}
	
	public synchronized void addSuffixServlet(Path suffix, Servlet servlet) {
		servletMap.putSuffix(suffix, servlet);
	}
	
	public synchronized void setDefaultServlet(Servlet servlet) {
		servletMap.setDefault(servlet);
	}
	
	public synchronized void addExactFilter(Path exactPath, Filter filter) {
		filterMap.putExact(exactPath, filter);
	}
	
	public synchronized void addPrefixFilter(Path prefix, Filter filter) {
		filterMap.putPrefix(prefix, filter);
	}
	
	public synchronized void addSuffixFilter(Path suffix, Filter filter) {
		filterMap.putSuffix(suffix, filter);
	}
	
	@Override
	protected synchronized void service(HttpServletRequest request,
			HttpServletResponse response) throws ServletException, IOException {
		
		FilterChain chain = EMPTY_CHAIN;
		Path path = ServletUtil.splitRequestPath(request.getPathInfo());
		ServletMatch servletMatch = servletMap.getMatch(path);
		
		if (servletMatch != null) {
			chain = new ServletMatchFilterChain(servletMatch);
		}
		
		List<Filter> filters = filterMap.getMatches(path);
		
		if (!filters.isEmpty()) {
			ListIterator<Filter> filterIter = filters.listIterator(filters.size());
			
			while (filterIter.hasPrevious()) {
				Filter filter = filterIter.previous();
				chain = new FilterFilterChain(filter, chain);
			}
		}
		chain.doFilter(request, response);
	}

}
