package org.apithefire.servlet.pathmap;

import java.util.Comparator;
import java.util.Map;
import java.util.SortedMap;
import java.util.TreeMap;

import javax.servlet.Servlet;

import org.apithefire.servlet.util.ServletUtil;
import org.apithefire.util.lang.Objects;
import org.apithefire.util.lang.Path;

class ServletMap {

	private static class LengthFirstPathComparator implements Comparator<Path> {

		public int compare(Path path1, Path path2) {
			int diff = path2.length() - path1.length();

			if (diff != 0) {
				return diff;
			}
			return path2.compareTo(path1);
		}

	}

	private static final Comparator<Path> LENGTH_FIRST_PATH_COMPARATOR =
		new LengthFirstPathComparator();
	
	private final SortedMap<Path, Servlet> exactServletMap;
	private final SortedMap<Path, Servlet> prefixServletMap;
	private final SortedMap<Path, Servlet> suffixServletMap;
	private Servlet defaultServlet;

	public ServletMap() {
		exactServletMap = new TreeMap<Path, Servlet>();
		prefixServletMap = new TreeMap<Path, Servlet>(LENGTH_FIRST_PATH_COMPARATOR);
		suffixServletMap = new TreeMap<Path, Servlet>(LENGTH_FIRST_PATH_COMPARATOR);
	}
	
	public ServletMatch getMatch(Path path) {
		Servlet servlet = exactServletMap.get(Objects.nonNull(path));
		
		if (servlet != null) {
			return new ServletMatch(servlet, path, Path.EMPTY_PATH);
		}
		
		ServletMatch servletMatch = getPrefixMatch(path);
		
		if (servletMatch != null) {
			return servletMatch;
		}
		
		servletMatch = getSuffixMatch(path);
		
		if (servletMatch != null) {
			return servletMatch;
		}
		if (defaultServlet == null) {
			return null;
		}
		return new ServletMatch(defaultServlet, path, Path.EMPTY_PATH);
	}
	
	private ServletMatch getPrefixMatch(Path path) {
		if (prefixServletMap.isEmpty()) {
			return null;
		}
		
		Map<Path, Servlet> tailMap = prefixServletMap.tailMap(path);
		
		if (tailMap.isEmpty()) {
			return null;
		}
		
		Map.Entry<Path, Servlet> firstMatch = tailMap.entrySet().iterator().next();
		Path prefix = firstMatch.getKey();
		Servlet servlet = firstMatch.getValue();
		
		if (!path.startsWith(prefix)) {
			return null;
		}
		
		Path servletPath = path.subPath(0, prefix.length());
		Path pathInfoPath = path.subPath(prefix.length());
		
		return new ServletMatch(servlet, servletPath, pathInfoPath);
	}
	
	private ServletMatch getSuffixMatch(Path path) {
		Path suffix = ServletUtil.splitLastComponentAsSuffix(path);
		
		if (suffix == null) {
			return null;
		}
		
		Map<Path, Servlet> tailMap = suffixServletMap.tailMap(suffix);
		
		if (tailMap.isEmpty()) {
			return null;
		}
		
		Map.Entry<Path, Servlet> firstMatch = tailMap.entrySet().iterator().next();
		Path matchSuffix = firstMatch.getKey();
		Servlet servlet = firstMatch.getValue();
		
		if (!suffix.endsWith(matchSuffix)) {
			return null;
		}
		return new ServletMatch(servlet, path, Path.EMPTY_PATH);
	}
	
	public void putExact(Path exactPath, Servlet servlet) {
		exactServletMap.put(Objects.nonNull(exactPath), Objects.nonNull(servlet));
	}
	
	public void putPrefix(Path prefix, Servlet servlet) {
		prefixServletMap.put(Objects.nonNull(prefix), Objects.nonNull(servlet));
	}
	
	public void putSuffix(Path suffix, Servlet servlet) {
		suffixServletMap.put(Objects.nonNull(suffix), Objects.nonNull(servlet));
	}
	
	public void setDefault(Servlet servlet) {
		defaultServlet = servlet;
	}

}
