package org.apithefire.servlet.resource;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.Date;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apithefire.servlet.util.ServletUtil;
import org.apithefire.util.lang.Objects;
import org.apithefire.util.lang.Path;
import org.apithefire.util.resource.Resource;
import org.apithefire.util.resource.ResourceNotFoundException;

public class ResourceServlet extends HttpServlet {
	
	public static final int DEFAULT_BUFFER_SIZE = 512;
	
	protected static final String HEADER_IF_MODIFIED_SINCE = "If-Modified-Since";
	protected static final String HEADER_LAST_MODIFIED = "Last-Modified";

	private Resource resource;
	private ContentTypeResolver contentTypeResolver;
	private int bufferSize = DEFAULT_BUFFER_SIZE;
	
	public ResourceServlet(Resource resource) {
		this.resource = Objects.nonNull(resource);
		this.contentTypeResolver = new DefaultContentTypeResolver();
	}
	
	public Resource getResource() {
		return resource;
	}

	public int getBufferSize() {
		return bufferSize;
	}

	public void setBufferSize(int bufferSize) {
		if (bufferSize < 1) {
			throw new IllegalArgumentException("Buffer size is less than one.");
		}
		this.bufferSize = bufferSize;
	}

	@Override
	protected void doGet(HttpServletRequest request, HttpServletResponse response)
			throws ServletException, IOException {
		
		Path pathInfo = ServletUtil.splitRequestPath(request.getPathInfo());
		String suffix = ServletUtil.parseSimpleSuffix(pathInfo);
		Resource relativeResource = null;
		
		try {
			relativeResource = resource.getResource(pathInfo);
		} catch (ResourceNotFoundException e) {
			response.sendError(HttpServletResponse.SC_NOT_FOUND);
			return;
		}
		if (!relativeResource.isContent()) {
			response.sendError(HttpServletResponse.SC_NOT_FOUND);
			return;
		}
		
		long lastCheckedTime = request.getDateHeader(HEADER_IF_MODIFIED_SINCE);
		Date lastModifiedTime = relativeResource.getContentModifiedTime();
		
		if (lastCheckedTime != -1 && lastModifiedTime != null) {
			if (lastModifiedTime.getTime() <= lastCheckedTime) {
				response.sendError(HttpServletResponse.SC_NOT_MODIFIED);
				return;
			}
		}
		if (lastModifiedTime != null) {
			response.setDateHeader(HEADER_LAST_MODIFIED, lastModifiedTime.getTime());
		}
		
		String contentType = contentTypeResolver.resolveContentType(suffix);
		
		response.setContentType(contentType);
		
		OutputStream out = response.getOutputStream();
		InputStream content = relativeResource.openContent();
		
		byte[] buffer = new byte[bufferSize];
		int n = content.read(buffer);
		
		while (n != -1) {
			out.write(buffer, 0, n);
			n = content.read(buffer);
		}
		response.flushBuffer();
	}
	
}
