package org.apithefire.servlet.server;

import java.io.IOException;
import java.net.ServerSocket;
import java.util.ArrayList;
import java.util.List;

import org.apithefire.util.lang.Objects;
import org.apithefire.util.lang.Path;
import org.apithefire.util.runtimeexception.io.RuntimeIoException;

public abstract class Server {
	
	private static int DEFAULT_CONNECTOR_PORT = 80;
	
	private final List<Connector> connectors = new ArrayList<Connector>();
	private final List<Context> contexts = new ArrayList<Context>();
	private Object serverImpl;

	private void checkNotStarted() {
		if (serverImpl != null) {
			throw new IllegalStateException("Server is started.");
		}
	}

	private void checkStarted() {
		if (serverImpl == null) {
			throw new IllegalStateException("Server is not started.");
		}
	}
	
	protected Object getServerImpl() {
		return serverImpl;
	}
	
	public void start() {
		checkNotStarted();
		
		if (connectors.isEmpty()) {
			throw new ServerException("No connector is added.");
		}
		if (contexts.isEmpty()) {
			throw new ServerException("No context is added");
		}
		serverImpl = createServerImpl();
		
		for (Connector connector : connectors) {
			connector.configure();
		}
		for (Context context : contexts) {
			context.configure();
		}
	}
	
	public void join() {
		checkStarted();
	}
	
	public void stop() {
		checkStarted();
	}
	
	public Connector addConnector() {
		return addConnector(DEFAULT_CONNECTOR_PORT);
	}

	// TODO: check duplicate port
	public Connector addConnector(int port) {
		checkNotStarted();
		
		Connector connector = Objects.nonNull(createConnector(port));
		
		connectors.add(connector);
		return connector;
	}
	
	public static int findUnusedPort() {
		ServerSocket socket = null;
		try {
			socket = new ServerSocket(0);
			return socket.getLocalPort();
		} catch (IOException e) {
			throw new RuntimeIoException(e);
		} finally {
			if (socket != null) {
				try {
					socket.close();
				} catch (IOException e) {
					throw new RuntimeIoException(e);
				}
			}
		}
	}
	
	public Context addContext() {
		return addContext(Path.EMPTY_PATH);
	}

	// TODO: check duplicate path
	public Context addContext(Path path) {
		checkNotStarted();
		
		Context context = Objects.nonNull(createContext(path));
		
		contexts.add(context);
		return context;
	}
	
	protected abstract Connector createConnector(int port);
	
	protected abstract Context createContext(Path path);
	
	protected abstract Object createServerImpl();
	
}
